import com.j256.ormlite.dao.Dao;
import com.j256.ormlite.dao.DaoManager;
import com.j256.ormlite.jdbc.JdbcConnectionSource;
import com.j256.ormlite.support.ConnectionSource;

import java.util.Scanner;

public class AccountApp {

        // we are using a MySQl database
        private final static String DATABASE_URL = "jdbc:mysql://localhost:3306/orm_lab?useUnicode=true";
        private String YOURUSERNAME= "root";
        private String YOURPASSWORD = "";
        String accountName = null;
        Account userSelection = null;

        private Dao<Account, Integer> accountDao;

        public static void main(String[] args) throws Exception {
            // turn our static method into an instance of Main
            new AccountApp().doMain(args);
        }

        private void doMain(String[] args) throws Exception {
            ConnectionSource connectionSource = null;
                try {
                    // create our data-source for the database
                    connectionSource = new JdbcConnectionSource(DATABASE_URL, YOURUSERNAME, YOURPASSWORD);
                    // setup our  DAOs
                    setupDao(connectionSource);
                    // read, write and delete some data
                    processData();

                    System.out.println("\n\nIt seems to have worked\n\n");
                } finally {
                    // destroy the data source which should close underlying connections

                    if (connectionSource != null) {
                        connectionSource.close();
                    }
                }

        }

        /**
         * Read and write some example data.
         */
        private void processData() throws Exception {
            boolean processing = true;
            do {
                switch (stringScan("Do You Want To Access An Account Or Create A New One? ACCESS/CREATE/EXIT to exit program")) {
                    case "create":
                        createAccount(stringScan("What Is The Account Name?"));
                        break;
                    case "access":
                        accessAccount();
                        break;
                    case "exit":
                        processing = false;
                        break;
                    default:
                        System.out.println("Invalid Choice!");
                        break;
                }
                } while (processing) ;
        }

        /**
         * Setup our  DAOs
         */
        private void setupDao(ConnectionSource connectionSource) throws Exception {

            accountDao = DaoManager.createDao(connectionSource, Account.class);

            }

            //scanners for user input
        private String stringScan(String prompt) {
            Scanner scan = new Scanner(System.in);
            String userInput = null;
            System.out.println(prompt);
            while (userInput == null) {
                if (scan.hasNext()) {
                    userInput = scan.nextLine();
                }
            } return userInput.toLowerCase();
        }

    public Integer intScan (String prompt) {
        Integer userInput = 0;
        System.out.println(prompt);
        while(userInput == 0) {
            Scanner integerIn = new Scanner(System.in);
            if (integerIn.hasNextInt()) {
                userInput = integerIn.nextInt();
            } else {
                System.out.println("Invalid Input! Please Try again!!");
            }
        }
        return userInput;
    }


        //creates new entry to table
        private void createAccount(String accountName) throws Exception {
            // create an instance of Account
            String name = accountName;
            Account account = new Account(name);
            // persist the account object to the database
            accountDao.create(account);
            int id = account.getId();
            System.out.println("Account id is: " + id);
            // assign a password
            account.setPassword(stringScan("Please Enter Password for New Account."));
            // update the database after changing the object
            accountDao.update(account);
        }

        //displays content of table
        private void readAccount() {
            accountDao.forEach(acc -> {
                System.out.println(acc.getId() + " : " + acc.getName() + " : " + acc.getPassword());
            });
        }

        // prompts user to update selected account, can change name and/or password
        private void updateAccount(int accountid) throws Exception {
            Account acc = accountDao.queryForId(accountid);
            boolean updating = true;
            while (updating) {
                switch (stringScan("Update NAME or PASSWORD or EXIT?")) {
                    case "name":
                        acc.setName(stringScan("What Is The Account's New Name?"));
                        accountDao.update(acc);
                        break;
                    case "password":
                        acc.setPassword(stringScan("What Is The Account's New Password?"));
                        accountDao.update(acc);
                        break;
                    case "exit": updating = false;
                        break;
                    default:
                        System.out.println("Invalid Choice!");
                        break;
                }
            }
        }


        //deletes selected account
        private void deleteAccount(int accountId) throws Exception {
            accountDao.deleteById(accountId);
        }

        private void accessAccount() throws Exception {
            readAccount();
            switch (stringScan("What Do You Want To Do To This Account? UPDATE/DELETE")) {
                case "update":
                    updateAccount( intScan("Please Pick The ID Number Of The Account You Want Access To."));
                    break;
                case "delete" :
                    deleteAccount( intScan("Please Pick The ID Number Of The Account You Want Access To."));
                    break;
                default:
                    System.out.println("Invalid Choice, Please Start Over!");
                    break;

            }

        }

    }

